@@ -1,7 +1,7 @@
 /*
  * detail.c	Process the detail file
  *
- * Version:	$Id: detail.c,v 1.10 2008/03/14 10:41:28 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: detail.c,v 1.10 2008/03/14 10:41:28 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -33,8 +33,14 @@ RCSID("$Id: detail.c,v 1.10 2008/03/14 10:41:28 aland Exp $")
 #include <sys/stat.h>
 #endif
 
+#ifdef HAVE_GLOB_H
+#include <glob.h>
+#endif
+
 #include <fcntl.h>
 
+#ifdef WITH_DETAIL
+
 #define USEC (1000000)
 
 typedef struct listen_detail_t {
@@ -84,7 +90,7 @@ int detail_send(rad_listen_t *listener, REQUEST *request)
 	 *	caller it's OK to read more "detail" file stuff.
 	 */
 	if (request->reply->code == 0) {
-		data->delay_time = 1;
+		data->delay_time = USEC;
 		data->signal = 1;
 		data->state = STATE_NO_REPLY;
 		radius_signal_self(RADIUS_SIGNAL_SELF_DETAIL);
@@ -192,6 +198,7 @@ static int detail_open(rad_listen_t *this)
 {
 	struct stat st;
 	listen_detail_t *data = this->data;
+	char *filename = data->filename;
 
 	rad_assert(data->state == STATE_UNOPENED);
 	data->delay_time = USEC;
@@ -205,10 +212,14 @@ static int detail_open(rad_listen_t *this)
 	 *	we've got to open it for writing in order to
 	 *	establish the lock, to prevent rlm_detail from
 	 *	writing to it.
+	 *
+	 *	This also means that if we're doing globbing,
+	 *	this file will be read && processed before the
+	 *	file globbing is done.
 	 */
 	this->fd = open(data->filename_work, O_RDWR);
 	if (this->fd < 0) {
-		DEBUG2("Polling for detail file %s", data->filename);
+		DEBUG2("Polling for detail file %s", filename);
 
 		/*
 		 *	Try reading the detail file.  If it
@@ -218,29 +229,64 @@ static int detail_open(rad_listen_t *this)
 		 *	exists, even if we don't have permissions
 		 *	to read it.
 		 */
-		if (stat(data->filename, &st) < 0) {
+		if (stat(filename, &st) < 0) {
+#ifdef HAVE_GLOB_H
+			int i, found;
+			time_t chtime;
+			glob_t files;
+
+			memset(&files, 0, sizeof(files));
+			if (glob(filename, 0, NULL, &files) != 0) {
+				return 0;
+			}
+
+			chtime = 0;
+			found = -1;
+			for (i = 0; i < files.gl_pathc; i++) {
+				if (stat(files.gl_pathv[i], &st) < 0) continue;
+
+				if ((i == 0) ||
+				    (st.st_ctime < chtime)) {
+					chtime = st.st_ctime;
+					found = i;
+				}
+			}
+
+			if (found < 0) {
+				globfree(&files);
+				return 0;
+			}
+
+			filename = strdup(files.gl_pathv[found]);
+			globfree(&files);
+#else
 			return 0;
+#endif
 		}
 
 		/*
 		 *	Open it BEFORE we rename it, just to
 		 *	be safe...
 		 */
-		this->fd = open(data->filename, O_RDWR);
+		this->fd = open(filename, O_RDWR);
 		if (this->fd < 0) {
 			radlog(L_ERR, "Failed to open %s: %s",
-			       data->filename, strerror(errno));
+			       filename, strerror(errno));
+			if (filename != data->filename) free(filename);
 			return 0;
 		}
 
 		/*
 		 *	Rename detail to detail.work
 		 */
-		if (rename(data->filename, data->filename_work) < 0) {
+		DEBUG("detail_recv: Renaming %s -> %s", filename, data->filename_work);
+		if (rename(filename, data->filename_work) < 0) {
+			if (filename != data->filename) free(filename);
 			close(this->fd);
 			this->fd = -1;
 			return 0;
 		}
+		if (filename != data->filename) free(filename);
 	} /* else detail.work existed, and we opened it */
 
 	rad_assert(data->vps == NULL);
@@ -378,7 +424,7 @@ int detail_recv(rad_listen_t *listener,
 			 *	we have.
 			 */
 		case STATE_READING:
-			if (!feof(data->fp)) break;
+			if (data->fp && !feof(data->fp)) break;
 			data->state = STATE_QUEUED;
 
 			/* FALL-THROUGH */
@@ -488,6 +534,14 @@ int detail_recv(rad_listen_t *listener,
 		 */
 		if (!strcasecmp(key, "Timestamp")) {
 			data->timestamp = atoi(value);
+
+			vp = paircreate(PW_PACKET_ORIGINAL_TIMESTAMP,
+					PW_TYPE_DATE);
+			if (vp) {
+				vp->vp_date = (uint32_t) data->timestamp;
+				*tail = vp;
+				tail = &(vp->next);
+			}
 			continue;
 		}
 
@@ -621,11 +675,9 @@ int detail_recv(rad_listen_t *listener,
 	*pfun = rad_accounting;
 
 	if (debug_flag) {
-		printf("detail_recv: Read packet from %s\n", data->filename);
+		fr_printf_log("detail_recv: Read packet from %s\n", data->filename_work);
 		for (vp = packet->vps; vp; vp = vp->next) {
-			putchar('\t');
-			vp_print(stdout, vp);
-			putchar('\n');
+			debug_pair(vp);
 		}
 	}
 
@@ -735,7 +787,31 @@ int detail_parse(CONF_SECTION *cs, rad_listen_t *this)
 		return -1;
 	}
 
-	snprintf(buffer, sizeof(buffer), "%s.work", data->filename);
+	/*
+	 *	If the filename is a glob, use "detail.work" as the
+	 *	work file name.
+	 */
+	if ((strchr(data->filename, '*') != NULL) ||
+	    (strchr(data->filename, '[') != NULL)) {
+		char *p;
+
+#ifndef HAVE_GLOB_H
+		radlog(L_INFO, "WARNING: Detail file \"%s\" appears to use file globbing, but it is not supported on this system.", data->filename);
+#endif
+		strlcpy(buffer, data->filename, sizeof(buffer));
+		p = strrchr(buffer, FR_DIR_SEP);
+		if (p) {
+			p[1] = '\0';
+		} else {
+			buffer[0] = '\0';
+		}
+		strlcat(buffer, "detail.work",
+			sizeof(buffer) - strlen(buffer));
+			
+	} else {
+		snprintf(buffer, sizeof(buffer), "%s.work", data->filename);
+	}
+
 	free(data->filename_work);
 	data->filename_work = strdup(buffer); /* FIXME: leaked */
 
@@ -757,3 +833,4 @@ int detail_parse(CONF_SECTION *cs, rad_listen_t *this)
 
 	return 0;
 }
+#endif
